@@ -1,11 +1,10 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond.pool import Pool
-from trytond.model import ModelView, ModelSQL, ModelSingleton, fields
-from trytond.transaction import Transaction
-from trytond.pyson import Eval, Id
+from trytond.model import ModelSingleton, ModelSQL, ModelView, fields
 from trytond.modules.company.model import (
     CompanyMultiValueMixin, CompanyValueMixin)
+from trytond.pool import Pool
+from trytond.pyson import Eval, Id
 
 tax_roundings = [
     ('document', 'Per Document'),
@@ -58,16 +57,42 @@ class Configuration(
                     Id('account',
                         'sequence_type_account_move_reconciliation')),
                 ]))
+    currency_exchange_journal = fields.MultiValue(fields.Many2One(
+            'account.journal', "Currency Exchange Journal",
+            domain=[
+                ('type', '=', 'write-off'),
+                ]))
+    currency_exchange_credit_account = fields.MultiValue(fields.Many2One(
+            'account.account', "Currency Exchange Credit Account",
+            domain=[
+                ('closed', '!=', True),
+                ('type.statement', '=', 'income'),
+                ('company', '=', Eval('context', {}).get('company', -1)),
+                ('second_currency', '=', None),
+                ]))
+    currency_exchange_debit_account = fields.MultiValue(fields.Many2One(
+            'account.account', "Currency Exchange Debit Account",
+            domain=[
+                ('closed', '!=', True),
+                ('type.statement', '=', 'income'),
+                ('company', '=', Eval('context', {}).get('company', -1)),
+                ('second_currency', '=', None),
+                ]))
 
     @classmethod
     def multivalue_model(cls, field):
         pool = Pool()
-        if field in {'default_account_receivable', 'default_account_payable'}:
+        if field in {
+                'default_account_receivable', 'default_account_payable',
+                'currency_exchange_credit_account',
+                'currency_exchange_debit_account'}:
             return pool.get('account.configuration.default_account')
         if field in {'default_customer_tax_rule', 'default_supplier_tax_rule'}:
             return pool.get('account.configuration.default_tax_rule')
         if field == 'reconciliation_sequence':
             return pool.get('account.configuration.sequence')
+        elif field == 'currency_exchange_journal':
+            return pool.get('account.configuration.journal')
         return super(Configuration, cls).multivalue_model(field)
 
     @classmethod
@@ -79,6 +104,11 @@ class Configuration(
         return cls.multivalue_model(
             'reconciliation_sequence').default_reconciliation_sequence()
 
+    @classmethod
+    def default_currency_exchange_journal(cls, **pattern):
+        return cls.multivalue_model(
+            'currency_exchange_journal').default_currency_exchange_journal()
+
 
 class ConfigurationDefaultAccount(ModelSQL, CompanyValueMixin):
     "Account Configuration Default Account"
@@ -89,16 +119,30 @@ class ConfigurationDefaultAccount(ModelSQL, CompanyValueMixin):
             ('type.receivable', '=', True),
             ('party_required', '=', True),
             ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
+            ])
     default_account_payable = fields.Many2One(
         'account.account', "Default Account Payable",
         domain=[
             ('type.payable', '=', True),
             ('party_required', '=', True),
             ('company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
+            ])
+    currency_exchange_credit_account = fields.Many2One(
+        'account.account', "Currency Exchange Credit Account",
+        domain=[
+            ('closed', '!=', True),
+            ('type.statement', '=', 'income'),
+            ('company', '=', Eval('company', -1)),
+            ('second_currency', '=', None),
+            ])
+    currency_exchange_debit_account = fields.Many2One(
+        'account.account', "Currency Exchange Debit Account",
+        domain=[
+            ('closed', '!=', True),
+            ('type.statement', '=', 'income'),
+            ('company', '=', Eval('company', -1)),
+            ('second_currency', '=', None),
+            ])
 
 
 class DefaultTaxRule(ModelSQL, CompanyValueMixin):
@@ -109,15 +153,13 @@ class DefaultTaxRule(ModelSQL, CompanyValueMixin):
         domain=[
             ('company', '=', Eval('company', -1)),
             ('kind', 'in', ['sale', 'both']),
-            ],
-        depends=['company'])
+            ])
     default_supplier_tax_rule = fields.Many2One(
         'account.tax.rule', "Default Supplier Tax Rule",
         domain=[
             ('company', '=', Eval('company', -1)),
             ('kind', 'in', ['purchase', 'both']),
-            ],
-        depends=['company'])
+            ])
 
 
 class ConfigurationTaxRounding(ModelSQL, CompanyValueMixin):
@@ -128,24 +170,9 @@ class ConfigurationTaxRounding(ModelSQL, CompanyValueMixin):
         context={
             'company': Eval('company', -1),
             },
-        depends=['company'])
+        depends={'company'})
     tax_rounding = fields.Selection(tax_roundings, 'Method', required=True)
 
-    @classmethod
-    def __register__(cls, module_name):
-        sql_table = cls.__table__()
-        cursor = Transaction().connection.cursor()
-
-        super(ConfigurationTaxRounding, cls).__register__(module_name)
-
-        table = cls.__table_handler__(module_name)
-
-        # Migration from 4.2: rename method into tax_rounding
-        if table.column_exist('method'):
-            cursor.execute(*sql_table.update(
-                    [sql_table.tax_rounding], [sql_table.method]))
-            table.drop_column('method')
-
     @classmethod
     def default_tax_rounding(cls):
         return 'document'
@@ -160,8 +187,7 @@ class Sequence(ModelSQL, CompanyValueMixin):
             ('company', 'in', [Eval('company', -1), None]),
             ('sequence_type', '=',
                 Id('account', 'sequence_type_account_move_reconciliation')),
-            ],
-        depends=['company'])
+            ])
 
     @classmethod
     def default_reconciliation_sequence(cls):
@@ -172,3 +198,25 @@ class Sequence(ModelSQL, CompanyValueMixin):
                 'account', 'sequence_account_move_reconciliation')
         except KeyError:
             return None
+
+
+class Journal(ModelSQL, CompanyValueMixin):
+    "Account Configuration Journal"
+    __name__ = 'account.configuration.journal'
+    currency_exchange_journal = fields.Many2One(
+        'account.journal', "Currency Exchange Journal",
+        domain=[
+            ('type', '=', 'write-off'),
+            ],
+        context={
+            'company': Eval('company', -1),
+            })
+
+    @classmethod
+    def default_currency_exchange_journal(cls):
+        pool = Pool()
+        ModelData = pool.get('ir.model.data')
+        try:
+            return ModelData.get_id('account', 'journal_currency_exchange')
+        except KeyError:
+            return None
