@@ -53,7 +53,11 @@ class AccountMissing(UserError):
     pass
 
 
-class SecondCurrencyError(ValidationError):
+class AccountValidationError(ValidationError):
+    pass
+
+
+class SecondCurrencyError(AccountValidationError):
     pass
 
 
@@ -65,7 +69,11 @@ class PostError(UserError):
     pass
 
 
-class MoveDatesError(ValidationError):
+class MoveTemplateExpressionError(UserError):
+    pass
+
+
+class MoveTemplateKeywordValidationError(ValidationError):
     pass
 
 
@@ -77,7 +85,7 @@ class ReconciliationError(ValidationError):
     pass
 
 
-class DeleteDelegatedWarning(UserWarning):
+class ReconciliationDeleteWarning(UserWarning):
     pass
 
 
@@ -87,3 +95,23 @@ class CancelDelegatedWarning(UserWarning):
 
 class GroupLineError(UserError):
     pass
+
+
+class GroupLineWarning(UserWarning):
+    pass
+
+
+class RescheduleLineError(UserError):
+    pass
+
+
+class RescheduleLineWarning(UserWarning):
+    pass
+
+
+class DelegateLineError(UserError):
+    pass
+
+
+class DelegateLineWarning(UserWarning):
+    pass
