@@ -1,18 +1,23 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from xml.sax.saxutils import quoteattr
 from decimal import Decimal
+from xml.sax.saxutils import quoteattr
 
-from simpleeval import simple_eval
+from simpleeval import InvalidExpression, simple_eval
 
+from trytond.i18n import gettext
 from trytond.model import (
-    ModelSQL, ModelView, DeactivableMixin, fields, sequence_ordered)
-from trytond.pyson import Eval
-from trytond.wizard import (Wizard, StateView, StateAction, StateTransition,
-    Button)
-from trytond.transaction import Transaction
+    DeactivableMixin, ModelSQL, ModelView, fields, sequence_ordered)
 from trytond.pool import Pool
+from trytond.pyson import Eval
 from trytond.tools import decistmt
+from trytond.transaction import Transaction
+from trytond.wizard import (
+    Button, StateAction, StateTransition, StateView, Wizard)
+
+from .exceptions import (
+    MoveTemplateExpressionError, MoveTemplateKeywordValidationError,
+    PeriodNotFoundError)
 
 
 class MoveTemplate(DeactivableMixin, ModelSQL, ModelView):
@@ -27,16 +32,14 @@ class MoveTemplate(DeactivableMixin, ModelSQL, ModelView):
         context={
             'company': Eval('company', -1),
             },
-        depends=['company'])
-    date = fields.Char('Date', help='Leave empty for today.')
+        depends={'company'})
     description = fields.Char('Description',
-        help="Keyword values substitutions are identified "
+        help="Keyword value substitutions are identified "
         "by braces ('{' and '}').")
     lines = fields.One2Many('account.move.line.template', 'move', 'Lines',
         domain=[
             ('account.company', '=', Eval('company', -1)),
-            ],
-        depends=['company'])
+            ])
 
     @staticmethod
     def default_company():
@@ -51,8 +54,6 @@ class MoveTemplate(DeactivableMixin, ModelSQL, ModelView):
         move = Move()
         move.company = self.company
         move.journal = self.journal
-        if self.date:
-            move.date = values.get(self.date)
         if self.description:
             move.description = self.description.format(
                 **dict(Keyword.format_values(self, values)))
@@ -66,7 +67,8 @@ class MoveTemplateKeyword(sequence_ordered(), ModelSQL, ModelView):
     __name__ = 'account.move.template.keyword'
     name = fields.Char('Name', required=True)
     string = fields.Char('String', required=True, translate=True)
-    move = fields.Many2One('account.move.template', 'Move', required=True)
+    move = fields.Many2One(
+        'account.move.template', "Move", required=True, ondelete='CASCADE')
     type_ = fields.Selection([
             ('char', 'Char'),
             ('numeric', 'Numeric'),
@@ -77,13 +79,24 @@ class MoveTemplateKeyword(sequence_ordered(), ModelSQL, ModelView):
     digits = fields.Integer('Digits', states={
             'invisible': Eval('type_') != 'numeric',
             'required': Eval('type_') == 'numeric',
-            }, depends=['type_'])
+            })
 
     @classmethod
     def __setup__(cls):
         super().__setup__()
         cls.__access__.add('move')
 
+    @classmethod
+    def validate(cls, keywords):
+        for keyword in keywords:
+            keyword.check_name()
+
+    def check_name(self):
+        if self.name and not self.name.isidentifier():
+            raise MoveTemplateKeywordValidationError(
+                gettext('account.msg_name_not_valid',
+                    name=self.name))
+
     @staticmethod
     def default_required():
         return False
@@ -156,7 +169,8 @@ class MoveTemplateKeyword(sequence_ordered(), ModelSQL, ModelView):
 class MoveLineTemplate(ModelSQL, ModelView):
     'Account Move Line Template'
     __name__ = 'account.move.line.template'
-    move = fields.Many2One('account.move.template', 'Move', required=True)
+    move = fields.Many2One(
+        'account.move.template', "Move", required=True, ondelete='CASCADE')
     operation = fields.Selection([
             ('debit', 'Debit'),
             ('credit', 'Credit'),
@@ -174,12 +188,11 @@ class MoveLineTemplate(ModelSQL, ModelView):
             'required': Eval('party_required', False),
             'invisible': ~Eval('party_required', False),
             },
-        depends=['party_required'],
         help="The name of the 'Party' keyword.")
     party_required = fields.Function(fields.Boolean('Party Required'),
         'on_change_with_party_required')
     description = fields.Char('Description',
-        help="Keywords values substitutions are identified "
+        help="Keyword value substitutions are identified "
         "by braces ('{' and '}').")
     taxes = fields.One2Many('account.tax.line.template', 'line', 'Taxes')
 
@@ -201,8 +214,23 @@ class MoveLineTemplate(ModelSQL, ModelView):
         Keyword = pool.get('account.move.template.keyword')
 
         line = Line()
-        amount = simple_eval(decistmt(self.amount),
-            functions={'Decimal': Decimal}, names=values)
+        try:
+            amount = simple_eval(decistmt(self.amount),
+                functions={'Decimal': Decimal}, names=values)
+        except (InvalidExpression, SyntaxError) as e:
+            raise MoveTemplateExpressionError(
+                gettext('account.msg_move_template_invalid_expression',
+                    expression=values,
+                    template=self.move.rec_name,
+                    error=e)) from e
+
+        if not isinstance(amount, Decimal):
+            raise MoveTemplateExpressionError(
+                gettext('account.msg_move_template_expression_not_number',
+                    value=amount,
+                    expression=self.move.name,
+                    template=self.move.rec_name))
+
         amount = self.move.company.currency.round(amount)
         if self.operation == 'debit':
             line.debit = amount
@@ -212,8 +240,15 @@ class MoveLineTemplate(ModelSQL, ModelView):
         if self.party:
             line.party = values.get(self.party)
         if self.description:
-            line.description = self.description.format(
-                **dict(Keyword.format_values(self.move, values)))
+            try:
+                line.description = self.description.format(
+                    **dict(Keyword.format_values(self.move, values)))
+            except KeyError as e:
+                raise MoveTemplateExpressionError(
+                    gettext('account.msg_move_template_invalid_expression',
+                        expression=values,
+                        template=self.move.name,
+                        error=e)) from e
         line.tax_lines = [t.get_line(values) for t in self.taxes]
 
         return line
@@ -222,7 +257,9 @@ class MoveLineTemplate(ModelSQL, ModelView):
 class TaxLineTemplate(ModelSQL, ModelView):
     'Account Tax Line Template'
     __name__ = 'account.tax.line.template'
-    line = fields.Many2One('account.move.line.template', 'Line', required=True)
+    line = fields.Many2One(
+        'account.move.line.template', "Line",
+        required=True, ondelete='CASCADE')
     amount = fields.Char('Amount', required=True,
         help="A python expression that will be evaluated with the keywords.")
     type = fields.Selection([
@@ -234,30 +271,35 @@ class TaxLineTemplate(ModelSQL, ModelView):
         domain=[
             ('company', '=', Eval('_parent_line', {}
                     ).get('_parent_move', {}).get('company', -1)),
-            ])
+            ],
+        depends={'line'})
 
     @classmethod
     def __setup__(cls):
         super().__setup__()
         cls.__access__.add('line')
 
-    @classmethod
-    def __register__(cls, module_name):
-        super(TaxLineTemplate, cls).__register__(module_name)
-
-        table_h = cls.__table_handler__(module_name)
-
-        # Migration from 4.6: remove code
-        table_h.drop_column('code')
-
     def get_line(self, values):
         'Return the tax line for the keyword values'
         pool = Pool()
         TaxLine = pool.get('account.tax.line')
 
         line = TaxLine()
-        amount = simple_eval(decistmt(self.amount),
-            functions={'Decimal': Decimal}, names=values)
+        try:
+            amount = simple_eval(decistmt(self.amount),
+                functions={'Decimal': Decimal}, names=values)
+        except (InvalidExpression, SyntaxError) as e:
+            raise MoveTemplateExpressionError(
+                gettext('account.msg_template_invalid_expression',
+                    expression=values,
+                    template=self.line.rec_name,
+                    error=e)) from e
+
+        if not isinstance(amount, Decimal):
+            raise MoveTemplateExpressionError(
+                gettext('account.msg_not_number',
+                    result=amount,
+                    expression=self.move.rec_name))
         amount = self.line.move.company.currency.round(amount)
         line.amount = amount
         line.type = self.type
@@ -317,6 +359,7 @@ class CreateMove(Wizard):
         for keyword in template.keywords:
             values[keyword.name] = getattr(self.keywords, keyword.name, None)
         move = template.get_move(values)
+        move.date = self.template.date
         move.period = self.template.period
         move.save()
         return move
@@ -342,8 +385,7 @@ class CreateMove(Wizard):
 
     def do_open_(self, action):
         move = self.create_move()
-        action['res_id'] = [move.id]
-        return action, {}
+        return action, {'res_id': move.id}
 
     def end(self):
         if self.model and self.model.__name__ == 'account.move.line':
@@ -358,19 +400,63 @@ class CreateMoveTemplate(ModelView):
         domain=[
             ('company', '=', Eval('context', {}).get('company', -1)),
             ])
+    date = fields.Date('Effective Date', required=True)
     period = fields.Many2One('account.period', 'Period', required=True,
         domain=[
-            ('state', '!=', 'close'),
+            ('state', '!=', 'closed'),
             ('fiscalyear.company.id', '=',
                 Eval('context', {}).get('company', 0)),
             ])
 
-    @staticmethod
-    def default_period():
+    @classmethod
+    def default_date(cls):
+        pool = Pool()
+        Date = pool.get('ir.date')
+        return Date.today()
+
+    @classmethod
+    def default_period(cls):
         pool = Pool()
         Period = pool.get('account.period')
         company = Transaction().context.get('company')
-        return Period.find(company, exception=False)
+        try:
+            period = Period.find(company)
+        except PeriodNotFoundError:
+            return None
+        return period.id
+
+    @fields.depends('date', 'period')
+    def on_change_date(self):
+        pool = Pool()
+        Period = pool.get('account.period')
+        company = Transaction().context.get('company')
+        if self.date:
+            if (not self.period
+                or not (
+                    self.period.start_date <= self.date
+                    <= self.period.end_date)):
+                try:
+                    self.period = Period.find(company, date=self.date)
+                except PeriodNotFoundError:
+                    pass
+
+    @fields.depends('period', 'date')
+    def on_change_period(self):
+        pool = Pool()
+        Date = pool.get('ir.date')
+        today = Date.today()
+        if self.period:
+            start_date = self.period.start_date
+            end_date = self.period.end_date
+            if (not self.date
+                    or not (start_date <= self.date <= end_date)):
+                if start_date <= today:
+                    if today <= end_date:
+                        self.date = today
+                    else:
+                        self.date = end_date
+                else:
+                    self.date = start_date
 
 
 class CreateMoveKeywords(ModelView):
