@@ -1,21 +1,22 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 
+from trytond.cache import Cache
+from trytond.const import OPERATORS
 from trytond.i18n import gettext
-from trytond.model import ModelView, ModelSQL, Workflow, fields
+from trytond.model import Index, ModelSQL, ModelView, Workflow, fields
 from trytond.model.exceptions import AccessError
+from trytond.pool import Pool
 from trytond.pyson import Eval, Id
 from trytond.transaction import Transaction
-from trytond.pool import Pool
-from trytond.const import OPERATORS
 
-from .exceptions import (PeriodNotFoundError, ClosePeriodError,
-    PeriodDatesError, PeriodSequenceError)
+from .exceptions import (
+    ClosePeriodError, PeriodDatesError, PeriodNotFoundError,
+    PeriodSequenceError)
 
 _STATES = {
     'readonly': Eval('state') != 'open',
 }
-_DEPENDS = ['state']
 
 
 class Period(Workflow, ModelSQL, ModelView):
@@ -23,18 +24,16 @@ class Period(Workflow, ModelSQL, ModelView):
     __name__ = 'account.period'
     name = fields.Char('Name', required=True)
     start_date = fields.Date('Starting Date', required=True, states=_STATES,
-        domain=[('start_date', '<=', Eval('end_date', None))],
-        depends=_DEPENDS + ['end_date'], select=True)
+        domain=[('start_date', '<=', Eval('end_date', None))])
     end_date = fields.Date('Ending Date', required=True, states=_STATES,
-        domain=[('end_date', '>=', Eval('start_date', None))],
-        depends=_DEPENDS + ['start_date'], select=True)
-    fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
-        required=True, states=_STATES, depends=_DEPENDS, select=True)
+        domain=[('end_date', '>=', Eval('start_date', None))])
+    fiscalyear = fields.Many2One(
+        'account.fiscalyear', "Fiscal Year", required=True, states=_STATES)
     state = fields.Selection([
             ('open', 'Open'),
-            ('close', 'Close'),
+            ('closed', 'Closed'),
             ('locked', 'Locked'),
-            ], 'State', readonly=True, required=True)
+            ], 'State', readonly=True, required=True, sort=False)
     post_move_sequence = fields.Many2One('ir.sequence', 'Post Move Sequence',
         domain=[
             ('sequence_type', '=',
@@ -43,26 +42,33 @@ class Period(Workflow, ModelSQL, ModelView):
                 ('company', '=', None),
                 ('company', '=', Eval('company', -1)),
                 ],
-            ],
-        depends=['company'])
+            ])
     type = fields.Selection([
             ('standard', 'Standard'),
             ('adjustment', 'Adjustment'),
             ], 'Type', required=True,
-        states=_STATES, depends=_DEPENDS, select=True)
+        states=_STATES)
     company = fields.Function(fields.Many2One('company.company', 'Company',),
         'on_change_with_company', searcher='search_company')
     icon = fields.Function(fields.Char("Icon"), 'get_icon')
+    _find_cache = Cache(__name__ + '.find', context=False)
 
     @classmethod
     def __setup__(cls):
         super(Period, cls).__setup__()
+        t = cls.__table__()
         cls.__access__.add('fiscalyear')
-        cls._order.insert(0, ('start_date', 'ASC'))
+        cls._sql_indexes.add(
+            Index(
+                t,
+                (t.start_date, Index.Range()),
+                (t.end_date, Index.Range()),
+                order='DESC'))
+        cls._order.insert(0, ('start_date', 'DESC'))
         cls._transitions |= set((
-                ('open', 'close'),
-                ('close', 'locked'),
-                ('close', 'open'),
+                ('open', 'closed'),
+                ('closed', 'locked'),
+                ('closed', 'open'),
                 ))
         cls._buttons.update({
                 'close': {
@@ -70,15 +76,24 @@ class Period(Workflow, ModelSQL, ModelView):
                     'depends': ['state'],
                     },
                 'reopen': {
-                    'invisible': Eval('state') != 'close',
+                    'invisible': Eval('state') != 'closed',
                     'depends': ['state'],
                     },
-                'lock': {
-                    'invisible': Eval('state') != 'close',
+                'lock_': {
+                    'invisible': Eval('state') != 'closed',
                     'depends': ['state'],
                     },
                 })
 
+    @classmethod
+    def __register__(cls, module):
+        cursor = Transaction().connection.cursor()
+        t = cls.__table__()
+        super().__register__(module)
+        # Migration from 6.8: rename state close to closed
+        cursor.execute(
+            *t.update([t.state], ['closed'], where=t.state == 'close'))
+
     @staticmethod
     def default_state():
         return 'open'
@@ -89,8 +104,7 @@ class Period(Workflow, ModelSQL, ModelView):
 
     @fields.depends('fiscalyear', '_parent_fiscalyear.company')
     def on_change_with_company(self, name=None):
-        if self.fiscalyear:
-            return self.fiscalyear.company.id
+        return self.fiscalyear.company if self.fiscalyear else None
 
     @classmethod
     def search_company(cls, name, clause):
@@ -99,98 +113,134 @@ class Period(Workflow, ModelSQL, ModelView):
     def get_icon(self, name):
         return {
             'open': 'tryton-account-open',
-            'close': 'tryton-account-close',
+            'closed': 'tryton-account-close',
             'locked': 'tryton-account-block',
             }.get(self.state)
 
     @classmethod
-    def validate(cls, periods):
-        super(Period, cls).validate(periods)
-        for period in periods:
-            period.check_dates()
-            period.check_fiscalyear_dates()
-            period.check_post_move_sequence()
-
-    def check_dates(self):
-        if self.type != 'standard':
-            return True
-        transaction = Transaction()
-        connection = transaction.connection
-        transaction.database.lock(connection, self._table)
-        table = self.__table__()
-        cursor = connection.cursor()
-        cursor.execute(*table.select(table.id,
-                where=(((table.start_date <= self.start_date)
-                        & (table.end_date >= self.start_date))
-                    | ((table.start_date <= self.end_date)
-                        & (table.end_date >= self.end_date))
-                    | ((table.start_date >= self.start_date)
-                        & (table.end_date <= self.end_date)))
-                & (table.fiscalyear == self.fiscalyear.id)
-                & (table.type == 'standard')
-                & (table.id != self.id)))
-        period_id = cursor.fetchone()
-        if period_id:
-            overlapping_period = self.__class__(period_id[0])
-            raise PeriodDatesError(
-                gettext('account.msg_period_overlap',
-                    first=self.rec_name,
-                    second=overlapping_period.rec_name))
-
-    def check_fiscalyear_dates(self):
-        if (self.start_date < self.fiscalyear.start_date
-                or self.end_date > self.fiscalyear.end_date):
-            raise PeriodDatesError(
-                gettext('account.msg_period_fiscalyear_dates',
-                    period=self.rec_name,
-                    fiscalyear=self.fiscalyear.rec_name))
-
-    def check_post_move_sequence(self):
-        if not self.post_move_sequence:
-            return
-        periods = self.search([
-                ('post_move_sequence', '=', self.post_move_sequence.id),
-                ('fiscalyear', '!=', self.fiscalyear.id),
-                ])
-        if periods:
-            raise PeriodSequenceError(
-                gettext('account.msg_period_same_sequence',
-                    first=self.rec_name,
-                    second=periods[0].rec_name))
+    def validate_fields(cls, periods, field_names):
+        super().validate_fields(periods, field_names)
+        cls.check_dates(periods, field_names)
+        cls.check_fiscalyear_dates(periods, field_names)
+        cls.check_post_move_sequence(periods, field_names)
 
     @classmethod
-    def find(cls, company_id, date=None, exception=True, test_state=True):
+    def check_dates(cls, periods, field_names=None):
+        if field_names and not (
+                field_names & {
+                    'start_date', 'end_date', 'fiscalyear', 'type'}):
+            return
+        transaction = Transaction()
+        connection = transaction.connection
+        cls.lock()
+        table = cls.__table__()
+        cursor = connection.cursor()
+        for period in periods:
+            if period.type != 'standard':
+                continue
+            cursor.execute(*table.select(table.id,
+                    where=(((table.start_date <= period.start_date)
+                            & (table.end_date >= period.start_date))
+                        | ((table.start_date <= period.end_date)
+                            & (table.end_date >= period.end_date))
+                        | ((table.start_date >= period.start_date)
+                            & (table.end_date <= period.end_date)))
+                    & (table.fiscalyear == period.fiscalyear.id)
+                    & (table.type == 'standard')
+                    & (table.id != period.id)))
+            period_id = cursor.fetchone()
+            if period_id:
+                overlapping_period = cls(period_id[0])
+                raise PeriodDatesError(
+                    gettext('account.msg_period_overlap',
+                        first=period.rec_name,
+                        second=overlapping_period.rec_name))
+
+    @classmethod
+    def check_fiscalyear_dates(cls, periods, field_names=None):
+        if field_names and not (
+                field_names & {
+                    'start_date', 'end_date', 'fiscalyear'}):
+            return
+        for period in periods:
+            fiscalyear = period.fiscalyear
+            if (period.start_date < fiscalyear.start_date
+                    or period.end_date > fiscalyear.end_date):
+                raise PeriodDatesError(
+                    gettext('account.msg_period_fiscalyear_dates',
+                        period=period.rec_name,
+                        fiscalyear=fiscalyear.rec_name))
+
+    @classmethod
+    def check_post_move_sequence(cls, periods, field_names=None):
+        if field_names and not (
+                field_names & {'post_move_sequence', 'fiscalyear'}):
+            return
+        for period in periods:
+            if not period.post_move_sequence:
+                continue
+            periods = cls.search([
+                    ('post_move_sequence', '=', period.post_move_sequence.id),
+                    ('fiscalyear', '!=', period.fiscalyear.id),
+                    ])
+            if periods:
+                raise PeriodSequenceError(
+                    gettext('account.msg_period_same_sequence',
+                        first=period.rec_name,
+                        second=periods[0].rec_name))
+
+    @classmethod
+    def find(cls, company, date=None, test_state=True):
         '''
-        Return the period for the company_id
-            at the date or the current date.
-        If exception is set the function will raise an exception
-            if no period is found.
-        If test_state is true, it will search on non-closed periods
+        Return the period for the company at the date or the current date
+        or raise PeriodNotFoundError.
+        If test_state is true, it searches on non-closed periods
         '''
         pool = Pool()
         Date = pool.get('ir.date')
         Lang = pool.get('ir.lang')
+        Company = pool.get('company.company')
 
+        company_id = int(company) if company is not None else None
         if not date:
-            date = Date.today()
-        clause = [
-            ('start_date', '<=', date),
-            ('end_date', '>=', date),
-            ('fiscalyear.company', '=', company_id),
-            ('type', '=', 'standard'),
-            ]
-        if test_state:
-            clause.append(('state', '=', 'open'))
-        periods = cls.search(clause, order=[('start_date', 'DESC')], limit=1)
-        if not periods:
-            if exception:
-                lang = Lang.get()
+            with Transaction().set_context(company=company_id):
+                date = Date.today()
+        key = (company_id, date, test_state)
+        period = cls._find_cache.get(key, -1)
+        if period is not None and period < 0:
+            clause = [
+                ('start_date', '<=', date),
+                ('end_date', '>=', date),
+                ('fiscalyear.company', '=', company_id),
+                ('type', '=', 'standard'),
+                ]
+            periods = cls.search(
+                clause, order=[('start_date', 'DESC')], limit=1)
+            if periods:
+                period, = periods
+            else:
+                period = None
+            cls._find_cache.set(key, int(period) if period else None)
+        elif period is not None:
+            period = cls(period)
+        found = period and (not test_state or period.state == 'open')
+        if not found:
+            lang = Lang.get()
+            if company is not None and not isinstance(company, Company):
+                company = Company(company)
+            if not period:
                 raise PeriodNotFoundError(
                     gettext('account.msg_no_period_date',
-                        date=lang.strftime(date)))
+                        date=lang.strftime(date),
+                        company=company.rec_name if company else ''))
             else:
-                return None
-        return periods[0].id
+                raise PeriodNotFoundError(
+                    gettext('account.msg_no_open_period_date',
+                        date=lang.strftime(date),
+                        period=period.rec_name,
+                        company=company.rec_name if company else ''))
+        else:
+            return period
 
     @classmethod
     def _check(cls, periods):
@@ -245,7 +295,9 @@ class Period(Workflow, ModelSQL, ModelView):
                 if not vals.get('post_move_sequence'):
                     vals['post_move_sequence'] = (
                         fiscalyear.post_move_sequence.id)
-        return super(Period, cls).create(vlist)
+        periods = super(Period, cls).create(vlist)
+        cls._find_cache.clear()
+        return periods
 
     @classmethod
     def write(cls, *args):
@@ -285,26 +337,43 @@ class Period(Workflow, ModelSQL, ModelView):
                                     period=period.rec_name))
             args.extend((periods, values))
         super(Period, cls).write(*args)
+        cls._find_cache.clear()
 
     @classmethod
     def delete(cls, periods):
         cls._check(periods)
         super(Period, cls).delete(periods)
+        cls._find_cache.clear()
 
     @classmethod
     @ModelView.button
-    @Workflow.transition('close')
+    @Workflow.transition('closed')
     def close(cls, periods):
         pool = Pool()
         JournalPeriod = pool.get('account.journal.period')
         Move = pool.get('account.move')
+        Account = pool.get('account.account')
         transaction = Transaction()
-        database = transaction.database
-        connection = transaction.connection
 
         # Lock period and move to be sure no new record will be created
-        database.lock(connection, JournalPeriod._table)
-        database.lock(connection, Move._table)
+        JournalPeriod.lock()
+        Move.lock()
+
+        for period in periods:
+            with transaction.set_context(
+                    fiscalyear=period.fiscalyear.id, date=period.end_date,
+                    cumulate=True, journal=None):
+                for account in Account.search([
+                            ('company', '=', period.company.id),
+                            ('end_date', '>=', period.start_date),
+                            ('end_date', '<=', period.end_date),
+                            ]):
+                    if account.balance:
+                        raise ClosePeriodError(
+                            gettext('account.'
+                                'msg_close_period_inactive_accounts',
+                                account=account.rec_name,
+                                period=period.rec_name))
 
         unposted_moves = Move.search([
                 ('period', 'in', [p.id for p in periods]),
@@ -331,7 +400,7 @@ class Period(Workflow, ModelSQL, ModelView):
     @classmethod
     @ModelView.button
     @Workflow.transition('locked')
-    def lock(cls, periods):
+    def lock_(cls, periods):
         pass
 
     @property
